
; option rom base file
; S. Adolph, J. Hogerhuis
; no support for FF45 control


;-----------------------------------------------------------------------
; M100 specific variables
;-----------------------------------------------------------------------



#include        "M100rom_published_lib.asm"
#include        "M100rom_unpublished_lib.asm"

OPON:			.EQU	0FAA4H


;-----------------------------------------------------------------------
;Globals on the Telcom Back Page to temporarily hold HL and
;DE registers during a CALL to Standard ROM, or during an
;Interrupt.
;-----------------------------------------------------------------------






;-----------------------------------------------------------------------
; ROM start
;-----------------------------------------------------------------------
	.org	0000h

RST0:	di
	JMP	program		; standard entry
	.DB	0,0,0,0

RST1:	RET
	.DB	0,0,0,0,0,0,0


RST2:	RET			;Not used
	.DB	0,0,0,0,0,0,0
RST3:	RET			;Not Used
	.DB	0,0,0,0,0,0,0
RST4:	RET			;Not Used
	.DB	0,0,0
TRAP:	DI
	CALL	INTCALL
RST5:	RET			;Not Used
	.DB	0,0,0
RST55:	DI
	CALL	INTCALL
RST6:	JMP	STDCALL		;RST 6 used as short call to a Standard ROM routine.
	.DB	0
RST65:	DI			;Replaces the 6.5 interrupt and sets up a call to 6.5 in Standard ROM
	CALL	INTCALL
RST7:	RET			;Not Used
	.DB	0,0,0
RST75: 	DI			;Replaces the 7.5 interrupt and sets up a call to 7.5 in Standard ROM
	CALL	INTCALL

	.org	0040h

OPON_img:			; auto copied on power up
				; 8 bytes
	PUSH	PSW		; temp store psw
	mvi	a,01h
	OUT	0E8H
	pop	psw	
	ret

	.db	00h




	.org	0048h

;---------------------------------------------------------
;The STDCALL routine allows a program running in the
;the Option ROM to call and return to an address in the
;Option ROM.
;Syntax is to use a RST 6 plus the address to be called.
;
; 		RST	6
; rvect-->	DW	04B44H  (cvect)
; pvect-->
;
;---------------------------------------------------------

STDCALL:			; 23 bytes
				; stack = rvect
	xthl
	inx 	h
	inx 	h
	xthl			; fix up first stack entry
				; stack = pvect
				; hl=hl'

	push	h			
	lxi 	h, OPON
	xthl			; stack = pvect, OPON
				; hl=hl'

	push 	h
	push 	d		; stack = pvect, OPON, hl', de'

	ldsi 	06h		; point de to stack location of return vector


	lhlx			; hl=pvect
	dcx 	h
	dcx 	h		; point hl=pvect-2
	xchg			; de=pvect-2
	lhlx			; hl=cvect

	pop 	d		; de=de'
	xthl			; hl=hl', stack= pvect, OPON, cvect

	jmp 	STDON


;---------------------------------------------------------
;Routine for the hardware traps.
;return address on entry is related to which interrupt occured.
; 
;		after interrupt occurs
; ivect		DI
;		call	INTCALL
; rvect -->
; so stack = pvect, rvect
;---------------------------------------------------------

INTCALL:			; final stack must be opon, introutine
				; interrupts are off
				; enter with return location on stack, indicating int routine
				; stack = pvect, rvect
				; 20 bytes
	push	h	
	push	d		; stack = pvect, rvect, hl', de'

	ldsi	04h

	lhlx			; hl= rvect

	dcx	h
	dcx	h
	dcx	h
	dcx	h		; hl = ivect

	push	h		; stack = pvect, rvect, hl', de', ivect

	lxi	h,OPON
	shlx			; stack = pvect, OPON, hl', de', ivect

	pop	h		; hl=ivect
	pop	d		; de=de'

	xthl			; stack = pvect, OPON, ivect
				; hl=hl', de=de'

	jmp	STDON

;---------------------------------------------------------
;OPX: Turns off the Option ROM (or turns on Standard ROM)
;Depending on your viewpoint.
;This routine must be ORGed at 85H. Ensure the the previous
;Code does not overlap into 85H
;
;---------------------------------------------------------
	.org	0085h

STDON:
	PUSH	PSW
	PUSH	H		; 26C8, F1 C9  POP PSW, RET
	LXI	H,26C8H		; it returns to this location --> pop psw; ret
	XTHL

OPEXIT:				; return to location pushed on stack
	xra	a
	OUT	0E8H
	RET			; RET can be found at 008EH in stdrom
				; in both M100 and T200


EXIT:
	.org	00094H
	pop	h
	lxi	h,0000h
	xthl			; restart

	jmp	OPEXIT




;-----------------------------------------------------------
;program starts here
;-----------------------------------------------------------

program:
		

	rst	6
	.dw	CLS			; display screen titles

print_msg:
	lxi	h,msg
print_loop:	
	push	h
	rst	6
	.dw	KYREAD
	pop	h
	jnz	EXIT

	mov 	a,m
	cpi	00h
	jz	delay
	rst	6
	.dw	LCD
	inx	h
	jmp	print_loop

delay:	lxi	h,8000h

delay1: dcx	h
	mov	a,h			; delay loop
	ora	l
	jnz	delay1

	jmp	print_msg



msg:	.db	"press any key to quit.......      ",00h


		
;--------------------------------------------------------------------
lastbyte:

	.fill 8000h-lastbyte,0FFh
	.END


